import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { AsyncSubject as AsyncSubject$1 } from 'rxjs/AsyncSubject';
import { Inject, InjectionToken, NgModule, Pipe } from '@angular/core';
import { CommonModule } from '@angular/common';
import { createStore } from 'redux';

const METADATA_KEY$1 = Symbol('@ReduxActionController');
const METADATA_DEFAULT$1 = {
    actionPrefix: '',
};
/**
 * @param {?} actionPrefix
 * @return {?}
 */
function ReduxActionController(actionPrefix) {
    return (constructor) => {
        const /** @type {?} */ metadata = { actionPrefix };
        Reflect['defineMetadata'](METADATA_KEY$1, metadata, constructor);
        return constructor;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxActionControllerMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$1, Reflect['getMetadata'](METADATA_KEY$1, target));
}

/**
 * @param {?} action
 * @return {?}
 */
function dispatch(action) {
    ReduxRegistry.getStore().then(store => store.dispatch(action));
}
const METADATA_KEY = Symbol('@ReduxActionController');
const METADATA_DEFAULT = {
    methodType: '',
    targetConstructor: null,
};
/**
 * @param {?=} customType
 * @return {?}
 */
function ReduxAction(customType) {
    return (target, propertyKey, descriptor) => {
        const /** @type {?} */ methodType = customType || propertyKey;
        const /** @type {?} */ method = descriptor.value;
        const /** @type {?} */ proxyFunction = function () {
            const /** @type {?} */ metadata = getReduxActionControllerMetadata(target.constructor);
            const /** @type {?} */ type = getActionType(metadata.actionPrefix, methodType);
            const /** @type {?} */ payload = method.apply(this, arguments);
            if (payload instanceof Promise) {
                payload.then((p) => dispatch({ payload: p, type }));
            }
            else {
                dispatch({ payload, type });
            }
            return payload;
        };
        Reflect['defineMetadata'](METADATA_KEY, {
            methodType,
            targetConstructor: target.constructor,
        }, proxyFunction);
        descriptor.value = proxyFunction;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxActionMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT, Reflect['getMetadata'](METADATA_KEY, target));
}
/**
 * @param {?} type
 * @return {?}
 */
function getActionTypeByFunction(type) {
    const { methodType, targetConstructor } = getReduxActionMetadata(type);
    const { actionPrefix } = getReduxActionControllerMetadata(targetConstructor);
    return getActionType(actionPrefix, methodType);
}
/**
 * @param {...?} rest
 * @return {?}
 */
function getActionType(...rest) {
    return rest.filter((i) => i.length > 0).join('/');
}

class ReduxRegistryReducerItem {
}
class ReduxRegistry {
    /**
     * @param {?} store
     * @return {?}
     */
    static registerStore(store) {
        ReduxRegistry._store.next(store);
        ReduxRegistry._store.complete();
    }
    /**
     * @param {?} stateName
     * @param {?} types
     * @param {?} reducer
     * @return {?}
     */
    static registerReducer(stateName, types, reducer) {
        if (Array.isArray(types)) {
            types.forEach((type) => {
                ReduxRegistry._reducers.push({
                    reducer,
                    stateName,
                    type,
                });
            });
        }
        else {
            ReduxRegistry._reducers.push({
                reducer,
                stateName,
                type: /** @type {?} */ (types),
            });
        }
    }
    /**
     * @param {?} stateName
     * @param {?} module
     * @return {?}
     */
    static registerModule(stateName, module) {
        ReduxRegistry.getStore().then((store) => {
            let /** @type {?} */ initialState = {};
            if (module.onReduxInit) {
                initialState = module.onReduxInit() || {};
            }
            store.dispatch({
                payload: {
                    initialState,
                    stateName,
                },
                type: ReduxRegistry.ACTION_REGISTER_MODULE,
            });
        });
    }
    /**
     * @param {?} type
     * @return {?}
     */
    static getReducerItemsByType(type) {
        return ReduxRegistry._reducers
            .filter(reducerItem => getActionTypeByFunction(reducerItem.type) === type);
    }
    /**
     * @return {?}
     */
    static getStore() {
        return new Promise(ReduxRegistry._store.subscribe.bind(ReduxRegistry._store));
    }
}
ReduxRegistry.ACTION_REGISTER_MODULE = `${ReduxRegistry.name}://registerModule`;
ReduxRegistry._store = new AsyncSubject$1();
ReduxRegistry._reducers = [];

/**
 * @template S
 * @param {?} state
 * @param {?} selector
 * @return {?}
 */
function selectByState(state, selector) {
    const /** @type {?} */ selectorArray = selector.split('/');
    if (selectorArray[0] !== '') {
        throw new Error('Relative selectors are not supported yet!');
    }
    return selectorArray
        .filter((propertyKey) => propertyKey !== '')
        .reduce((previousValue, propertyKey) => {
        if (!previousValue || !previousValue[propertyKey]) {
            return null;
        }
        return previousValue[propertyKey];
    }, state);
}
/**
 * @template S
 * @param {?} selector
 * @return {?}
 */
function select(selector) {
    const /** @type {?} */ subject = new BehaviorSubject$1(null);
    ReduxRegistry.getStore().then(store => {
        subject.next(selectByState(store.getState(), selector));
        store.subscribe(() => subject.next(selectByState(store.getState(), selector)));
    });
    return subject.asObservable();
}

const METADATA_KEY$2 = Symbol('@ReduxActionController');
const METADATA_DEFAULT$2 = {
    reducers: [],
};
/**
 * @template S, P
 * @param {?} types
 * @return {?}
 */
function ReduxReducer(types) {
    return (target, propertyKey) => {
        const /** @type {?} */ metadata = getReduxReducerClassMetadata(target.constructor);
        metadata.reducers.push({ types, reducer: target[propertyKey].bind(target) });
        Reflect['defineMetadata'](METADATA_KEY$2, metadata, target.constructor);
        if (typeof target === 'object') {
            target[propertyKey].bind(target);
        }
        if (!target['__@ReduxReducer']) {
            target['__@ReduxReducer'] = [];
        }
        return target[propertyKey];
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxReducerClassMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$2, Reflect['getMetadata'](METADATA_KEY$2, target));
}

/**
 * @param {?=} config
 * @return {?}
 */
function Redux(config) {
    return (constructor) => {
        return class extends constructor {
            constructor() {
                super(...arguments);
                this.redux = (() => {
                    const /** @type {?} */ mergedConfig = Object.assign({
                        reducers: [],
                        stateName: '',
                    }, config || {});
                    const { stateName, reducers } = mergedConfig;
                    // constructor[ '__@Redux' ] = mergedConfig;
                    ReduxRegistry.registerModule(stateName, this);
                    reducers
                        .map((reducer) => getReduxReducerClassMetadata(reducer.constructor))
                        .forEach((metadata) => {
                        metadata.reducers.forEach(reducer => {
                            ReduxRegistry.registerReducer(stateName, reducer.types, reducer.reducer);
                        });
                    });
                    return config;
                })();
            }
        };
    };
}

/**
 * @param {?} selector
 * @return {?}
 */
function ReduxSelect(selector) {
    return (target, propertyKey) => {
        Object.defineProperty(target, propertyKey, {
            configurable: true,
            enumerable: true,
            value: select(selector),
        });
    };
}

class ReduxSelectPipe {
    /**
     * @param {?} selector
     * @return {?}
     */
    transform(selector) {
        return select(selector);
    }
}
ReduxSelectPipe.decorators = [
    { type: Pipe, args: [{
                name: 'reduxSelect',
            },] },
];
/**
 * @nocollapse
 */
ReduxSelectPipe.ctorParameters = () => [];

const rootReducer = (state, action) => {
    if (action.type === ReduxRegistry.ACTION_REGISTER_MODULE) {
        const { stateName, initialState } = action.payload;
        return Object.assign({}, state, {
            [stateName]: initialState,
        });
    }
    return ReduxRegistry.getReducerItemsByType(action.type)
        .reduce((stateToReduce, reducerItem) => {
        const { stateName, reducer } = reducerItem;
        return Object.assign({}, stateToReduce, {
            [stateName]: reducer(stateToReduce[stateName], action),
        });
    }, state);
};

const REDUX_MODULE_CONFIG = new InjectionToken('REDUX_MODULE_CONFIG');
const IS_ROOT_MODULE = new InjectionToken('IS_ROOT_MODULE');
class ReduxModule {
    /**
     * @param {?=} isRootModule
     * @param {?=} config
     */
    constructor(isRootModule = false, config = null) {
        if (isRootModule) {
            config = Object.assign({
                store: null,
            }, config || {});
            ReduxRegistry.registerStore(config.store || createStore(rootReducer, {}));
        }
    }
    /**
     * @return {?}
     */
    static forChild() {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: IS_ROOT_MODULE, useValue: false },
            ],
        };
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: REDUX_MODULE_CONFIG, useValue: config },
                { provide: IS_ROOT_MODULE, useValue: true },
            ],
        };
    }
}
ReduxModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ReduxSelectPipe,
                ],
                exports: [
                    ReduxSelectPipe,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [],
            },] },
];
/**
 * @nocollapse
 */
ReduxModule.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [IS_ROOT_MODULE,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [REDUX_MODULE_CONFIG,] },] },
];

/**
 * Generated bundle index. Do not edit.
 */

export { selectByState, select, ReduxAction, getReduxActionMetadata, getActionTypeByFunction, getActionType, ReduxActionController, getReduxActionControllerMetadata, ReduxReducer, getReduxReducerClassMetadata, Redux, ReduxSelect, ReduxSelectPipe, REDUX_MODULE_CONFIG, IS_ROOT_MODULE, ReduxModule, ReduxRegistryReducerItem, ReduxRegistry, rootReducer };
//# sourceMappingURL=ngx-redux-core.js.map
